// Copyright (c) 2007 Henric Jungheim <software@henric.info>
//
// Permission to use, copy, modify, and distribute this software for any
// purpose with or without fee is hereby granted, provided that the above
// copyright notice and this permission notice appear in all copies.
//
// THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
// WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
// ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
// ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
// OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


namespace dSMFT
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class Window1 : System.Windows.Window
    {
        System.Diagnostics.Stopwatch sw = new System.Diagnostics.Stopwatch();
        PlatformMath.RandomSFMT rng = PlatformMath.RandomSFMT.CreateRandom();

        public Window1()
        {
            InitializeComponent();

            silly.Text = PlatformMath.Status.HasSSE2 ? "has" : "hasn't";

            platform.Text = Environment.OSVersion.Platform.ToString();

            architecture.Text = Environment.GetEnvironmentVariable("PROCESSOR_ARCHITECTURE");

            UpdateRandom(10);
        }

        private void UpdateRandom(int count)
        {
            if (null == rng)
            {
                moreInfo.Text = "No RNG available";
                return;
            }

            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < count; ++i)
                sb.AppendLine(rng.NextDouble().ToString("F8"));

            moreInfo.Text = sb.ToString();
        }

        void OnUpdateClick(object sender, RoutedEventArgs e)
        {
            UpdateRandom(10);
        }
#if false
        void OnWriteClick(object sender, RoutedEventArgs e)
        {
            System.Windows.Forms.FileDialog fd = new System.Windows.Forms.SaveFileDialog();

            if (fd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                using (System.IO.TextWriter tw = new System.IO.StreamWriter(fd.FileName))
                {
                    rng.Reset(1234);
                    tw.WriteLine(rng.Name);
                    for (int i = 0; i < 250; ++i)
                    {
                        tw.WriteLine("{0:F15} {1:F15} {2:F15} {3:F15}",
                            rng.NextDouble(), rng.NextDouble(), rng.NextDouble(), rng.NextDouble());
                    }
                }
            }
        }
#endif // false

        const int benchmarkCount = 10000000;
        void OnBenchmarkClick(object sender, RoutedEventArgs e)
        {
            long bestSMFT = long.MaxValue;
            long bestLib = long.MaxValue;

            System.Random libRng = new System.Random();

            for (int rep = 0; rep < 3; ++rep)
            {
                sw.Reset();
                sw.Start();

                for (int i = 0; i < benchmarkCount; ++i)
                    rng.NextDouble();

                sw.Stop();

                if (sw.ElapsedMilliseconds < bestSMFT)
                    bestSMFT = sw.ElapsedMilliseconds;

                sw.Reset();
                sw.Start();

                for (int i = 0; i < benchmarkCount; ++i)
                    libRng.NextDouble();

                sw.Stop();

                if (sw.ElapsedMilliseconds < bestLib)
                    bestLib = sw.ElapsedMilliseconds;
            }

            runtime.Text = string.Format("{0:G4} ns", 1000.0 * 1000.0 * bestSMFT / (double)benchmarkCount);
            libRuntime.Text = string.Format("{0:G4} ns", 1000.0 * 1000.0 * bestLib / (double)benchmarkCount);
        }
    }
}